@@ -2,7 +2,7 @@
 // GB_callback_proto.h: prototypes for functions for kernel callbacks
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2024, All Rights Reserved.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2025, All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
@@ -17,7 +17,7 @@
 #define GB_CALLBACK_PROTO_H
 
 #define GB_CALLBACK_SAXPY3_CUMSUM_PROTO(GX_AxB_saxpy3_cumsum)               \
-void GX_AxB_saxpy3_cumsum                                                   \
+GrB_Info GX_AxB_saxpy3_cumsum                                               \
 (                                                                           \
     GrB_Matrix C,               /* finalize C->p */                         \
     GB_saxpy3task_struct *SaxpyTasks, /* list of tasks, and workspace */    \
@@ -45,7 +45,7 @@ void GX_bitmap_M_scatter_whole  /* scatter M into the C bitmap */           \
 GrB_Info GX_bix_alloc       /* allocate A->b, A->i, and A->x in a matrix */ \
 (                                                                           \
     GrB_Matrix A,           /* matrix to allocate space for */              \
-    const GrB_Index nzmax,  /* number of entries the matrix can hold; */    \
+    const uint64_t nzmax,   /* number of entries the matrix can hold; */    \
                             /* ignored if A is iso and full */              \
     const int sparsity,     /* sparse (=hyper/auto) / bitmap / full */      \
     const bool bitmap_calloc,   /* if true, calloc A->b, else use malloc */ \
@@ -57,22 +57,10 @@ GrB_Info GX_bix_alloc       /* allocate A->b, A->i, and A->x in a matrix */ \
 void GX_ek_slice            /* slice a matrix */                            \
 (                                                                           \
     /* output: */                                                           \
-    int64_t *restrict A_ek_slicing, /* size 3*ntasks+1 */                   \
+    int64_t *restrict A_ek_slicing, /* size 3*A_ntasks+1 */                 \
     /* input: */                                                            \
     GrB_Matrix A,                   /* matrix to slice */                   \
-    int ntasks                      /* # of tasks */                        \
-)
-
-#define GB_CALLBACK_EK_SLICE_MERGE1_PROTO(GX_ek_slice_merge1)               \
-void GX_ek_slice_merge1     /* merge column counts for the matrix C */      \
-(                                                                           \
-    /* input/output: */                                                     \
-    int64_t *restrict Cp,               /* column counts */                 \
-    /* input: */                                                            \
-    const int64_t *restrict Wfirst,     /* size A_ntasks */                 \
-    const int64_t *restrict Wlast,      /* size A_ntasks */                 \
-    const int64_t *A_ek_slicing,        /* size 3*A_ntasks+1 */             \
-    const int A_ntasks                  /* # of tasks */                    \
+    int A_ntasks                    /* # of tasks */                        \
 )
 
 #define GB_CALLBACK_FREE_MEMORY_PROTO(GX_free_memory)                       \
@@ -144,11 +132,13 @@ GrB_Info GX_subassign_one_slice     /* slice M for subassign_05, 06n, 07 */ \
     int *p_nthreads,                /* # of threads to use */               \
     /* input: */                                                            \
     const GrB_Matrix C,             /* output matrix C */                   \
-    const GrB_Index *I,                                                     \
+    const void *I,                                                          \
+    const bool I_is_32,                                                     \
     const int64_t nI,                                                       \
     const int Ikind,                                                        \
     const int64_t Icolon [3],                                               \
-    const GrB_Index *J,                                                     \
+    const void *J,                                                          \
+    const bool J_is_32,                                                     \
     const int64_t nJ,                                                       \
     const int Jkind,                                                        \
     const int64_t Jcolon [3],                                               \
@@ -160,7 +150,7 @@ GrB_Info GX_subassign_one_slice     /* slice M for subassign_05, 06n, 07 */ \
 GrB_Info GX_add_phase0          /* find vectors in C for C=A+B or C<M>=A+B*/\
 (                                                                           \
     int64_t *p_Cnvec,           /* # of vectors to compute in C */          \
-    int64_t *restrict *Ch_handle,        /* Ch: size Cnvec, or NULL */      \
+    void **Ch_handle,           /* Ch: size Cnvec, or NULL */               \
     size_t *Ch_size_handle,              /* size of Ch in bytes */          \
     int64_t *restrict *C_to_M_handle,    /* C_to_M: size Cnvec, or NULL */  \
     size_t *C_to_M_size_handle,          /* size of C_to_M in bytes */      \
@@ -169,6 +159,9 @@ GrB_Info GX_add_phase0          /* find vectors in C for C=A+B or C<M>=A+B*/\
     int64_t *restrict *C_to_B_handle,    /* C_to_B: size Cnvec, or NULL */  \
     size_t *C_to_B_size_handle,          /* size of C_to_A in bytes */      \
     bool *p_Ch_is_Mh,           /* if true, then Ch == Mh */                \
+    bool *p_Cp_is_32,           /* if true, Cp is 32-bit; else 64-bit */    \
+    bool *p_Cj_is_32,           /* if true, Ch is 32-bit; else 64-bit */    \
+    bool *p_Ci_is_32,           /* if true, Ci is 32-bit; else 64-bit */    \
     int *C_sparsity,            /* sparsity structure of C */               \
     const GrB_Matrix M,         /* optional mask, may be NULL; not compl */ \
     const GrB_Matrix A,         /* first input matrix */                    \
@@ -186,7 +179,8 @@ GrB_Info GX_ewise_slice                                                     \
     int *p_nthreads,                /* # of threads for eWise operation */  \
     /* input: */                                                            \
     const int64_t Cnvec,            /* # of vectors of C */                 \
-    const int64_t *restrict Ch,     /* vectors of C, if hypersparse */      \
+    const void *Ch,                 /* vectors of C, if hypersparse */      \
+    const bool Cj_is_32,            /* if true, Ch is 32-bit, else 64-bit */\
     const int64_t *restrict C_to_M, /* mapping of C to M */                 \
     const int64_t *restrict C_to_A, /* mapping of C to A */                 \
     const int64_t *restrict C_to_B, /* mapping of C to B */                 \
@@ -214,11 +208,10 @@ GrB_Info GX_subassign_IxJ_slice                                             \
 #define GB_CALLBACK_PENDING_ENSURE_PROTO(GX_Pending_ensure)                 \
 bool GX_Pending_ensure                                                      \
 (                                                                           \
-    GB_Pending *PHandle,    /* input/output */                              \
+    GrB_Matrix C,           /* matrix with C->Pending */                    \
     bool iso,               /* if true, do not allocate Pending->x */       \
     GrB_Type type,          /* type of pending tuples */                    \
     GrB_BinaryOp op,        /* operator for assembling pending tuples */    \
-    bool is_matrix,         /* true if Pending->j must be allocated */      \
     int64_t nnew,           /* # of pending tuples to add */                \
     GB_Werk Werk                                                            \
 )
@@ -232,23 +225,26 @@ GrB_Info GX_subassign_08n_slice                                             \
     int *p_ntasks,                  /* # of tasks constructed */            \
     int *p_nthreads,                /* # of threads to use */               \
     int64_t *p_Znvec,               /* # of vectors to compute in Z */      \
-    const int64_t *restrict *Zh_handle,  /* Zh is A->h, M->h, or NULL */    \
-    int64_t *restrict *Z_to_A_handle,    /* Z_to_A: size Znvec, or NULL */  \
+    const void **Zh_handle,         /* Zh is A->h, M->h, or NULL */         \
+    int64_t *restrict *Z_to_A_handle, /* Z_to_A: size Znvec, or NULL */     \
     size_t *Z_to_A_size_handle,                                             \
-    int64_t *restrict *Z_to_M_handle,    /* Z_to_M: size Znvec, or NULL */  \
+    int64_t *restrict *Z_to_M_handle, /* Z_to_M: size Znvec, or NULL */     \
     size_t *Z_to_M_size_handle,                                             \
+    bool *Zj_is_32_handle,                                                  \
     /* input: */                                                            \
-    const GrB_Matrix C,             /* output matrix C */                   \
-    const GrB_Index *I,                                                     \
+    const GrB_Matrix C,         /* output matrix C */                       \
+    const void *I,              /* I index list */                          \
+    const bool I_is_32,                                                     \
     const int64_t nI,                                                       \
     const int Ikind,                                                        \
     const int64_t Icolon [3],                                               \
-    const GrB_Index *J,                                                     \
+    const void *J,              /* J index list */                          \
+    const bool J_is_32,                                                     \
     const int64_t nJ,                                                       \
     const int Jkind,                                                        \
     const int64_t Jcolon [3],                                               \
-    const GrB_Matrix A,             /* matrix to slice */                   \
-    const GrB_Matrix M,             /* matrix to slice */                   \
+    const GrB_Matrix A,         /* matrix to slice */                       \
+    const GrB_Matrix M,         /* matrix to slice */                       \
     GB_Werk Werk                                                            \
 )
 
@@ -259,22 +255,16 @@ void GX_bitmap_assign_to_full   /* set all C->b to 1, or make C full */     \
     int nthreads_max                                                        \
 )
 
-#define GB_CALLBACK_QSORT_1_PROTO(GX_qsort_1)                               \
-void GX_qsort_1    /* sort array A of size 1-by-n */                        \
-(                                                                           \
-    int64_t *restrict A_0,      /* size n array */                          \
-    const int64_t n                                                         \
-)
-
 #define GB_CALLBACK_P_SLICE_PROTO(GX_p_slice)                               \
-void GX_p_slice                     /* slice Ap */                          \
+void GX_p_slice                 /* slice Ap, 32-bit or 64-bit */            \
 (                                                                           \
     /* output: */                                                           \
-    int64_t *restrict Slice,        /* size ntasks+1 */                     \
+    int64_t *restrict Slice,    /* size ntasks+1 */                         \
     /* input: */                                                            \
-    const int64_t *restrict Ap,     /* array size n+1 (full/bitmap: NULL)*/ \
+    const void *Work,           /* array size n+1 (full/bitmap: NULL)*/     \
+    bool Work_is_32,            /* if true, Ap is uint32_t, else uint64_t */\
     const int64_t n,                                                        \
-    const int ntasks,               /* # of tasks */                        \
+    const int ntasks,           /* # of tasks */                            \
     const bool perfectly_balanced                                           \
 )
 
